{
   21/12/97.


   - TbhDate component, a descendant of TMaskEdit, facilitates the entering
     and retrieving of Australian date with ease.

   - Year 2000 compliance.

  
}
unit bhDate;

interface

uses
  Wintypes, Winprocs, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Mask,
  MyDate;

const
  DefaultDateWidth   : byte = 100;
  DefaultDateEditMask: string[18] = 'cc!\/cc\/cccc;1;_';
  DefaultDateText    : string[11] = '  /  /    ';

  InvalidDateMsg     : string[26] = 'Invalid Australian date!';

type
  TbhDate = class(TMaskEdit)
    public
      constructor Create(AOwner : TComponent); override;

      function SetDateByString(const Value : string) : boolean;
      procedure SetDateByDate(const Value : TDateTime);
      procedure SetDateByJulianDate(const Value : JulianDate);

      function GetDateAsString : string;
      function GetDateAsDate : TDateTime;
      function GetDateAsJulianDate : JulianDate;

      function DateIsValid(dstr : string) : boolean;
      
    protected
      procedure KeyPress(var Key: Char); override;

    private
      procedure CMEnter(var Msg: TCMEnter); message CM_ENTER;
      procedure CMExit(var Msg: TCMExit); message CM_EXIT;
  end;

procedure Register;

implementation
uses MyString;


constructor TbhDate.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);

  EditMask := DefaultDateEditMask;
  Text     := DefaultDateText;
  Width    := DefaultDateWidth;

  ShortDateFormat := 'dd/MM/yyyy';
end;



function TbhDate.SetDateByString(const Value : string) : boolean;
begin
  SetDateByString := False;
  
  self.Text := Value;

  if (DateIsValid(self.Text)) then
    SetDateByString := true;
end;

{
  21/12/97.

  Value is supposed to always contain a valid date. The value that
  constant input variable Value contains should always be defined
  by the Delphi StrToDate function.
}
procedure TbhDate.SetDateByDate(const Value : TDateTime);
begin
  {Extra pre-caution.}
  try
    self.Text := DateToStr(Value);
  except
    on EConvertError do
      ;
  end;
  DateIsValid(self.Text);
end;


procedure TbhDate.SetDateByJulianDate(const Value : JulianDate);
var
  D, M, Y : integer;
  s1,s2: string[2];
begin
  MyDate .DateToDMY(Value, D, M, Y);
  s1:=IntToStr(D);
  if length(s1) = 1 then
    s1:= '0' + s1;
   s2 := IntToStr(M);
  if length(s2) = 1 then
    s2:= '0' + s2;
  self.Text := s1 + '/' + s2 + '/' + IntToStr(Y);
end;

function TbhDate.GetDateAsString : string;
var
  s : string;
begin
  GetDateAsString := '';

  if (DateIsValid(self.Text)) then
    begin
      s := Remove_Embeded_Spaces(self.Text);
      GetDateAsString := s;
    end;

end;

function TbhDate.GetDateAsDate : TDateTime;
var
  s : string;
begin
  GetDateAsDate := 0;

  if (DateIsValid(self.Text)) then
    begin
      s := Remove_Embeded_Spaces(self.Text);
      GetDateAsDate := StrToDate(s);
    end;
end;

function TbhDate.GetDateAsJulianDate : JulianDate;
var
  s : string;
  D, M, Y : integer;
begin
  GetDateAsJulianDate := 0;
  
  if (DateIsValid(self.Text)) then
    begin
      s := Remove_Embeded_Spaces(self.Text);
      
      MyDate .StrToDMY(s, D, M, Y);
      GetDateAsJulianDate := MyDate .DMYtoDate(D, M, Y);
    end;
end;

procedure TbhDate.KeyPress(var Key: Char);
begin
  if (not (Key in ['0'..'9', ' '])) then
    Key := #0;

  inherited KeyPress(Key);
end;


function TbhDate.DateIsValid( dstr : string) : boolean;
var
  s : string;
begin
  DateIsValid := true;

  s := Remove_Embeded_Spaces(dstr);

  try
    StrToDate(s);
  except
    on EConvertError do
      begin
        DateIsValid := False;
        MessageDlg(InvalidDateMsg, mtError, [mbOk], 0);
        SetFocus;
      end;
  end;  {try}
  
end;

procedure TbhDate.CMEnter(var Msg: TCMEnter);
begin
  EditMask := DefaultDateEditMask;
  Width    := DefaultDateWidth;

  inherited;
end;


procedure TbhDate.CMExit(var Msg: TCMExit);
begin
  DateIsValid(self.Text);

  inherited;
end;

procedure Register;
begin
  RegisterComponents('Custom', [TbhDate]);
end;

end.
